@@ -13,6 +13,7 @@ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLI
 """
 
 import atexit
+import os
 import platform
 import random
 import signal
@@ -27,9 +28,16 @@ import sentry_sdk
 from sentry_sdk._compat import PY33
 from sentry_sdk._queue import Queue
 from sentry_sdk._types import MYPY
-from sentry_sdk.utils import nanosecond_time
+from sentry_sdk.utils import (
+    filename_for_module,
+    handle_in_app_impl,
+    logger,
+    nanosecond_time,
+)
 
-RawFrameData = namedtuple("RawFrameData", ["function", "abs_path", "lineno"])
+RawFrameData = namedtuple(
+    "RawFrameData", ["abs_path", "filename", "function", "lineno", "module"]
+)
 
 if MYPY:
     from types import FrameType
@@ -45,7 +53,7 @@ if MYPY:
     from typing_extensions import TypedDict
     import sentry_sdk.tracing
 
-    RawSampleData = Tuple[int, Sequence[Tuple[int, Sequence[RawFrameData]]]]
+    RawSampleData = Tuple[int, Sequence[Tuple[str, Sequence[RawFrameData]]]]
 
     ProcessedStack = Tuple[int, ...]
 
@@ -61,9 +69,11 @@ if MYPY:
     ProcessedFrame = TypedDict(
         "ProcessedFrame",
         {
+            "abs_path": str,
+            "filename": Optional[str],
             "function": str,
-            "filename": str,
             "lineno": int,
+            "module": Optional[str],
         },
     )
 
@@ -83,7 +93,6 @@ if MYPY:
     )
 
 
-_sample_buffer = None  # type: Optional[SampleBuffer]
 _scheduler = None  # type: Optional[Scheduler]
 
 
@@ -94,33 +103,33 @@ def setup_profiler(options):
     `buffer_secs` determines the max time a sample will be buffered for
     `frequency` determines the number of samples to take per second (Hz)
     """
-    buffer_secs = 30
-    frequency = 101
+
+    global _scheduler
+
+    if _scheduler is not None:
+        logger.debug("profiling is already setup")
+        return
 
     if not PY33:
-        from sentry_sdk.utils import logger
-
         logger.warn("profiling is only supported on Python >= 3.3")
         return
 
-    global _sample_buffer
-    global _scheduler
-
-    assert _sample_buffer is None and _scheduler is None
+    buffer_secs = 30
+    frequency = 101
 
     # To buffer samples for `buffer_secs` at `frequency` Hz, we need
     # a capcity of `buffer_secs * frequency`.
-    _sample_buffer = SampleBuffer(capacity=buffer_secs * frequency)
+    buffer = SampleBuffer(capacity=buffer_secs * frequency)
 
-    profiler_mode = options["_experiments"].get("profiler_mode", SigprofScheduler.mode)
+    profiler_mode = options["_experiments"].get("profiler_mode", SleepScheduler.mode)
     if profiler_mode == SigprofScheduler.mode:
-        _scheduler = SigprofScheduler(sample_buffer=_sample_buffer, frequency=frequency)
+        _scheduler = SigprofScheduler(sample_buffer=buffer, frequency=frequency)
     elif profiler_mode == SigalrmScheduler.mode:
-        _scheduler = SigalrmScheduler(sample_buffer=_sample_buffer, frequency=frequency)
+        _scheduler = SigalrmScheduler(sample_buffer=buffer, frequency=frequency)
     elif profiler_mode == SleepScheduler.mode:
-        _scheduler = SleepScheduler(sample_buffer=_sample_buffer, frequency=frequency)
+        _scheduler = SleepScheduler(sample_buffer=buffer, frequency=frequency)
     elif profiler_mode == EventScheduler.mode:
-        _scheduler = EventScheduler(sample_buffer=_sample_buffer, frequency=frequency)
+        _scheduler = EventScheduler(sample_buffer=buffer, frequency=frequency)
     else:
         raise ValueError("Unknown profiler mode: {}".format(profiler_mode))
     _scheduler.setup()
@@ -131,13 +140,11 @@ def setup_profiler(options):
 def teardown_profiler():
     # type: () -> None
 
-    global _sample_buffer
     global _scheduler
 
     if _scheduler is not None:
         _scheduler.teardown()
 
-    _sample_buffer = None
     _scheduler = None
 
 
@@ -162,14 +169,25 @@ def extract_stack(frame, max_stack_depth=MAX_STACK_DEPTH):
         stack.append(frame)
         frame = frame.f_back
 
-    return [
-        RawFrameData(
-            function=get_frame_name(frame),
-            abs_path=frame.f_code.co_filename,
-            lineno=frame.f_lineno,
-        )
-        for frame in stack
-    ]
+    return tuple(extract_frame(frame) for frame in stack)
+
+
+def extract_frame(frame):
+    # type: (FrameType) -> RawFrameData
+    abs_path = frame.f_code.co_filename
+
+    try:
+        module = frame.f_globals["__name__"]
+    except Exception:
+        module = None
+
+    return RawFrameData(
+        abs_path=os.path.abspath(abs_path),
+        filename=filename_for_module(module, abs_path) or None,
+        function=get_frame_name(frame),
+        lineno=frame.f_lineno,
+        module=module,
+    )
 
 
 def get_frame_name(frame):
@@ -243,18 +261,24 @@ class Profile(object):
         self.scheduler.stop_profiling()
         self._stop_ns = nanosecond_time()
 
-    def to_json(self, event_opt):
-        # type: (Any) -> Dict[str, Any]
+    def to_json(self, event_opt, options):
+        # type: (Any, Dict[str, Any]) -> Dict[str, Any]
         assert self._start_ns is not None
         assert self._stop_ns is not None
 
+        profile = self.scheduler.sample_buffer.slice_profile(
+            self._start_ns, self._stop_ns
+        )
+
+        handle_in_app_impl(
+            profile["frames"], options["in_app_exclude"], options["in_app_include"]
+        )
+
         return {
             "environment": event_opt.get("environment"),
             "event_id": uuid.uuid4().hex,
             "platform": "python",
-            "profile": self.scheduler.sample_buffer.slice_profile(
-                self._start_ns, self._stop_ns
-            ),
+            "profile": profile,
             "release": event_opt.get("release", ""),
             "timestamp": event_opt["timestamp"],
             "version": "1",
@@ -324,7 +348,7 @@ class SampleBuffer(object):
     def slice_profile(self, start_ns, stop_ns):
         # type: (int, int) -> ProcessedProfile
         samples = []  # type: List[ProcessedSample]
-        stacks = dict()  # type: Dict[ProcessedStack, int]
+        stacks = dict()  # type: Dict[int, int]
         stacks_list = list()  # type: List[ProcessedStack]
         frames = dict()  # type: Dict[RawFrameData, int]
         frames_list = list()  # type: List[ProcessedFrame]
@@ -334,39 +358,46 @@ class SampleBuffer(object):
         #
         # Is it safe to assume that the samples are always in
         # chronological order and binary search the buffer?
-        for raw_sample in self.buffer:
-            if raw_sample is None:
-                continue
-
-            ts = raw_sample[0]
+        for ts, sample in filter(None, self.buffer):
             if start_ns > ts or ts > stop_ns:
                 continue
 
-            for tid, stack in raw_sample[1]:
-                current_stack = []
+            elapsed_since_start_ns = str(ts - start_ns)
 
-                for frame in stack:
-                    if frame not in frames:
-                        frames[frame] = len(frames)
-                        frames_list.append(
-                            {
-                                "function": frame.function,
-                                "filename": frame.abs_path,
-                                "lineno": frame.lineno,
-                            }
-                        )
-                    current_stack.append(frames[frame])
+            for tid, stack in sample:
+                # Instead of mapping the stack into frame ids and hashing
+                # that as a tuple, we can directly hash the stack.
+                # This saves us from having to generate yet another list.
+                # Additionally, using the stack as the key directly is
+                # costly because the stack can be large, so we pre-hash
+                # the stack, and use the hash as the key as this will be
+                # needed a few times to improve performance.
+                hashed_stack = hash(stack)
 
-                current_stack = tuple(current_stack)
-                if current_stack not in stacks:
-                    stacks[current_stack] = len(stacks)
-                    stacks_list.append(current_stack)
+                # Check if the stack is indexed first, this lets us skip
+                # indexing frames if it's not necessary
+                if hashed_stack not in stacks:
+                    for frame in stack:
+                        if frame not in frames:
+                            frames[frame] = len(frames)
+                            frames_list.append(
+                                {
+                                    "abs_path": frame.abs_path,
+                                    "function": frame.function or "<unknown>",
+                                    "filename": frame.filename,
+                                    "lineno": frame.lineno,
+                                    "module": frame.module,
+                                }
+                            )
+
+                    stacks[hashed_stack] = len(stacks)
+                    stacks_list.append(tuple(frames[frame] for frame in stack))
 
                 samples.append(
                     {
-                        "elapsed_since_start_ns": str(ts - start_ns),
-                        "thread_id": str(tid),
-                        "stack_id": stacks[current_stack],
+                        "elapsed_since_start_ns": elapsed_since_start_ns,
+                        "thread_id": tid,
+                        "stack_id": stacks[hashed_stack],
                     }
                 )
 
@@ -375,7 +406,7 @@ class SampleBuffer(object):
         # will not have any metadata associated with it.
         thread_metadata = {
             str(thread.ident): {
-                "name": thread.name,
+                "name": str(thread.name),
             }
             for thread in threading.enumerate()
         }  # type: Dict[str, ProcessedThreadMetadata]
@@ -401,7 +432,7 @@ class SampleBuffer(object):
                 (
                     nanosecond_time(),
                     [
-                        (tid, extract_stack(frame))
+                        (str(tid), extract_stack(frame))
                         for tid, frame in sys._current_frames().items()
                     ],
                 )
@@ -695,7 +726,7 @@ def _should_profile(transaction, hub):
         return False
 
     # The profiler hasn't been properly initialized.
-    if _sample_buffer is None or _scheduler is None:
+    if _scheduler is None:
         return False
 
     hub = hub or sentry_sdk.Hub.current
